"use strict";
/**
 * Copyright 2021 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.analyzeCommits = void 0;
var dayjs_1 = __importDefault(require("dayjs"));
var DEFAULT_CONFIG = {
    units: "week",
    duration: 1,
    release: "patch"
};
function analyzeCommits(_a, context) {
    var _b = _a.duration, duration = _b === void 0 ? DEFAULT_CONFIG.duration : _b, _c = _a.units, units = _c === void 0 ? DEFAULT_CONFIG.units : _c, _d = _a.release, release = _d === void 0 ? DEFAULT_CONFIG.release : _d;
    return __awaiter(this, void 0, void 0, function () {
        var nextRelease, commits, logger, filteredCommits, committerDates, firstCommitterDate, thresholdDate, isBefore;
        return __generator(this, function (_e) {
            nextRelease = context.nextRelease, commits = context.commits, logger = context.logger;
            if (nextRelease || !commits) {
                logger.log("Skipping interval based commit analysis since release already set.");
                return [2 /*return*/, null];
            }
            filteredCommits = commits.filter(function (_a) {
                var message = _a.message;
                return message.trim();
            });
            if (filteredCommits.length === 0) {
                logger.log("Skipping interval based commit analysis due to lack of valid commits.");
                return [2 /*return*/, null];
            }
            committerDates = filteredCommits
                .map(function (_a) {
                var committerDate = _a.committerDate;
                return committerDate;
            })
                .map(function (date) { return (0, dayjs_1["default"])(date); })
                .sort(function (a, b) { return (a.isAfter(b) ? 1 : -1); });
            committerDates.forEach(function (date) {
                logger.log("committerDate: ".concat(date.format()));
            });
            firstCommitterDate = committerDates[0];
            thresholdDate = (0, dayjs_1["default"])().subtract(duration, units);
            isBefore = firstCommitterDate.isBefore(thresholdDate);
            logger.log("Oldest `committerDate` ".concat(isBefore ? "before" : "after", " threshold, ").concat(duration, " ").concat(units, "(s).\n  oldest commit: ").concat(firstCommitterDate.format(), "\n  threshold:     ").concat(thresholdDate.format()));
            if (isBefore) {
                logger.log("Setting release to '".concat(release, "' based upon interval."));
                return [2 /*return*/, release];
            }
            return [2 /*return*/, null];
        });
    });
}
exports.analyzeCommits = analyzeCommits;
